@@ -85,7 +85,7 @@ setConstructorS3("Object", Object);
 #
 # \examples{
 #   obj <- Object()
-#   as.character(obj)    # "Object: 0x016A80FC"
+#   as.character(obj)    # "Object: 0x000000000ab6a7a8"
 # }
 #
 # @author
@@ -100,46 +100,8 @@ setConstructorS3("Object", Object);
 setMethodS3("as.character", "Object", function(x, ...) {
   # To please R CMD check
   this <- x;
-
-  intToHex <- function(x, width=NULL) {
-    y <- as.integer(x)
-    class(y) <- "hexmode"
-    y <- as.character(y)
-    if (!is.null(width)) {
-      missing <- width - nchar(y);
-      for (k in which(missing > 0)) {
-        zeros <- paste(rep("0", missing[k]), collapse="");
-        y[k] <- paste(zeros, y[k], sep="");
-      }
-    }
-    dim(y) <- dim(x)
-    y
-  }
-
-  as.character.hexmode <- function(x) {
-    hexDigit <- c(0:9, "A", "B", "C", "D", "E", "F");
-    isna <- is.na(x);
-    y <- x[!isna];
-    ans0 <- character(length(y));
-    z <- NULL;
-    while (any(y > 0) | is.null(z)) {
-      z <- y%%16;
-      y <- floor(y/16);
-      ans0 <- paste(hexDigit[z + 1], ans0, sep = "");
-    }
-    ans <- rep(NA, length(x));
-    ans[!isna] <- ans0;
-    ans
-  }
-
-  addr <- getInternalAddress(this);
-  hex <- c(addr %/% 2^32, addr %% 2^32);
-  hex[2] <- intToHex(hex[2], width=8);
-  hex[1] <- intToHex(hex[1]);
-  if (nchar(hex[1]) %% 2 == 1)
-    hex[1] <- paste("0", hex[1], sep="");
-  hex <- paste(hex, collapse="");
-  paste(class(this)[1], ": 0x", addr, sep="");
+  addr <- getInternalAddress(this, format="hexstring");
+  paste(class(this)[1], ": ", addr, sep="");
 }) # as.character()
 
 
@@ -356,38 +318,7 @@ setMethodS3("equals", "Object", function(this, other, ...) {
 #   \item{...}{Not used.}
 # }
 #
-# \examples{
-#   setConstructorS3("MyClass", function() {
-#     extend(Object(), "MyClass")
-#   })
-#
-#   setMethodS3("finalize", "MyClass", function(this) {
-#     cat(as.character(this), "is about to be removed from the memory!\n")
-#   })
-#
-#   o <- MyClass()
-#   o <- MyClass()
-#   o <- MyClass()
-#   o <- MyClass()
-#   gc()
-#
-#   \dontrun{
-#   MyClass: 0x01BE602C is about to be removed from the memory!
-#   MyClass: 0x01BFF634 is about to be removed from the memory!
-#   MyClass: 0x01C13584 is about to be removed from the memory!
-#            used (Mb) gc trigger (Mb)
-#   Ncells 229903  6.2     467875 12.5
-#   Vcells  53725  0.5     786432  6.0
-#   }
-#
-#   rm(o)
-#   \dontrun{
-#   MyClass: 0x01C578B0 is about to be removed from the memory!
-#            used (Mb) gc trigger (Mb)
-#   Ncells 229903  6.1     467875 12.3
-#   Vcells  53725  0.5     786432  6.0
-#   }
-# }
+# @examples "../incl/Object.finalize.Rex"
 #
 # @author
 #
@@ -455,7 +386,7 @@ setMethodS3("finalize", "Object", function(this, ...) {
 # \keyword{methods}
 #*/###########################################################################
 setMethodS3("hashCode", "Object", function(this, ...) {
-  getInternalAddress(this);
+  getInternalAddress(this, format="numeric");
 }) # hashCode()
 
 
@@ -476,17 +407,20 @@ setMethodS3("hashCode", "Object", function(this, ...) {
 # @synopsis
 #
 # \arguments{
+#   \item{format}{A @character string specifying what format to return.}
 #   \item{...}{Not used.}
 # }
 #
 # \value{
-#   Returns a @double (can hold 64-bit addresses, whereas an @integer can
-#   only hold 32-bit addresses).
+#   The address is returned as a @numeric integer if 
+#   \code{format == "numeric"}, and as a @character string if
+#   \code{format == "hexstring"}.
 # }
 #
 # \examples{
 #   obj <- Object()
-#   getInternalAddress(obj)    # 26979608
+#   getInternalAddress(obj, format="numeric")    # 179742632
+#   getInternalAddress(obj, format="hexstring")  # "0x000000000ab6a7a8"
 # }
 #
 # @author
@@ -499,7 +433,10 @@ setMethodS3("hashCode", "Object", function(this, ...) {
 # \keyword{programming}
 # \keyword{methods}
 #*/###########################################################################
-setMethodS3("getInternalAddress", "Object", function(this, ...) {
+setMethodS3("getInternalAddress", "Object", function(this, format=c("numeric", "hexstring"), ...) {
+  # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+  # Local functions
+  # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   hexStringToDouble <- function(hex) {
     hexDigits <- unlist(strsplit("0123456789ABCDEF", ""));
     digits16 <- unlist(strsplit(toupper(hex), ""));
@@ -512,9 +449,14 @@ setMethodS3("getInternalAddress", "Object", function(this, ...) {
     as.integer(hexStringToDouble(hex));
   }
 
+  # Argument 'format':
+  format <- match.arg(format);
+
   pointer <- getName(attr(this, ".env"));
-  pointer <- gsub("0x", "", pointer);
-  pointer <- hexStringToDouble(pointer);
+  if (format == "numeric") {
+    pointer <- gsub("0x", "", pointer);
+    pointer <- hexStringToDouble(pointer);
+  }
 
   pointer;
 }, private=TRUE) # getInternalAddress()
@@ -598,8 +540,9 @@ setMethodS3("print", "Object", function(x, ...) {
 # @keyword "programming"
 #*/#########################################################################
 setMethodS3("attachLocally", "Object", function(this, private=FALSE, fields=NULL, excludeFields=NULL, overwrite=TRUE, envir=parent.frame(), ...) {
-  if (is.null(fields))
+  if (is.null(fields)) {
     fields <- getFields(this, private=private);
+  }
   fields <- setdiff(fields, excludeFields);
 
   attachedFields <- c();
@@ -769,7 +712,7 @@ setMethodS3("detach", "Object", function(this, ...) {
 #*/###########################################################################
 setMethodS3("save", "Object", function(this, file=NULL, path=NULL, compress=TRUE, ..., safe=TRUE) {
   if (is.null(file)) {
-    file <- sprintf("%s.%d.RData", class(this)[1], getInternalAddress(this));
+    file <- sprintf("%s.%d.RData", class(this)[1], getInternalAddress(this, format="numeric"));
   } 
 
   # Saving to a file?
@@ -1014,7 +957,7 @@ setMethodS3("objectSize", "Object", function(this, ...) {
 setMethodS3("getStaticInstance", "Object", function(this, ...) {
   className <- class(this)[1];
   # WAS: clazz <- get(className);
-  clazz <- .getClassByName(className);
+  clazz <- .getClassByName(className, ...);
   getStaticInstance(clazz);
 }, protected=TRUE) # getStaticInstance()
 
@@ -1267,6 +1210,7 @@ setMethodS3("staticCode", "Object", function(static, ...) {
 #   \item{...}{Named values representing the fields of the new instance.}
 #   \item{...fields}{An optional named @list of fields.  This makes it possible
 #     to specify a set of fields using a @list object.}
+#   \item{...envir}{An @environment.}
 # }
 #
 # \value{
@@ -1303,7 +1247,7 @@ setMethodS3("staticCode", "Object", function(static, ...) {
 if (is.element("R.oo", search())) {
   rm("extend", pos="R.oo"); # To avoid warning about renaming existing extend()
 }
-setMethodS3("extend", "Object", function(this, ...className, ..., ...fields=NULL) {
+setMethodS3("extend", "Object", function(this, ...className, ..., ...fields=NULL, ...envir=parent.frame()) {
   # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   # Local functions
   # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -1343,13 +1287,13 @@ setMethodS3("extend", "Object", function(this, ...className, ..., ...fields=NULL
   fields <- parseModifiers(fields);
 
   names <- names(fields);
-  for (ii in seq(fields)) {
+  for (ii in seq(along=fields)) {
     name <- names[ii];
     if (is.null(name) || nchar(name) == 0) {
       callNames <- names(sys.call());
       callNames <- callNames[nchar(callNames) > 0];
       matchNames <- paste("^", callNames, sep="");
-      for (jj in seq(matchNames)) {
+      for (jj in seq(along=matchNames)) {
         if (regexpr(matchNames[jj], "...className") != -1) {
           className <- sys.call()[[3]];
           throw("Could not set field of class (probably called ", className, 
@@ -1368,62 +1312,17 @@ setMethodS3("extend", "Object", function(this, ...className, ..., ...fields=NULL
   # Note, we have to re-register the finalizer here and not in Object(), 
   # because here the reference variable 'this' will have the correct
   # class attribute, which it does not in Object().
-  # NOTE: The finalizer() depends on the 'this' object. # /HB 2011-04-02
-  finalizer <- function(env) {
-    # Note, R.oo might be detached when this is called!  If so, reload
-    # it, this will be our best chance to run the correct finalizer(),
-    # which might be in a subclass of a different package that is still
-    # loaded.
-    isRooLoaded <- any(is.element(c("package:R.oo", "dummy:R.oo"), search()));
-    if (isRooLoaded) {
-      finalize(this);
-    } else {
-      suppressMessages({
-        isRooLoaded <- require("R.oo", quietly=TRUE);
-      })
-
-      # For unknown reasons R.oo might not have been loaded.
-      if (isRooLoaded) {
-        finalize(this);
-      } else {
-        warning("Failed to temporarily reload 'R.oo' and finalize().");
-      }
-
-      # NOTE! Before detach R.oo again, we have to make sure the Object:s
-      # allocated by R.oo itself (e.g. an Package object), will not reload
-      # R.oo again when being garbage collected, resulting in an endless
-      # loop.  We do this by creating a dummy finalize() function, detach
-      # R.oo, call garbage collect to clean out all R.oo's objects, and
-      # then remove the dummy finalize() function.
-      # (1) Put a dummy finalize() function on the search path.
-      # To please R CMD check
-      attachX <- base::attach;
-      attachX(list(finalize = function(...) { }), name="dummy:R.oo",
-                                                    warn.conflicts=FALSE);
-      # (2) Detach R.oo
-      if (is.element("package:R.oo", search())) {
-        detach("package:R.oo");
-      }
-
-      # (3) Force all R.oo's Object:s to be finalize():ed.
-      gc();
-
-      # (4) Remove the dummy finalize():er again.
-      if (is.element("dummy:R.oo", search())) {
-        detach("dummy:R.oo");
-      }
-    }
-  } # finalizer()
-
+  finalizer <- .makeObjectFinalizer(this, reloadRoo=TRUE);
   onexit <- getOption("R.oo::Object/finalizeOnExit", FALSE);
   reg.finalizer(this.env, finalizer, onexit=onexit);
 
 
   # Finally, create the static instance?
   if (!is.element("Class", ...className)) {
-    static <- getStaticInstance(this);
-    if (!is.null(static))
+    static <- getStaticInstance(this, envir=...envir);
+    if (!is.null(static)) {
       staticCode(static);
+    }
   }
 
   # Record field modifiers
@@ -1512,11 +1411,26 @@ setMethodS3("extend", "Object", function(this, ...className, ..., ...fields=NULL
 # \keyword{methods}
 #*/###########################################################################
 setMethodS3("$", "Object", function(this, name) {
-  memberAccessorOrder <- attr(this, ".memberAccessorOrder");
-  if (is.null(memberAccessorOrder))
-    memberAccessorOrder <- c(1,2,3,4,5);
+  # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+  # Local functions
+  # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+  getStatic <- function(static=NULL) {
+    if (!is.null(static)) return(static);
+    className <- class(this)[1];
+    clazz <- .getClassByName(className, envir=environment(this));
+    static <- getStaticInstance(clazz);
+    static;
+  } # getStatic()
 
-  # Check if there is a cache lookup available.
+
+  memberAccessorOrder <- attr(this, ".memberAccessorOrder");
+  if (is.null(memberAccessorOrder)) {
+    memberAccessorOrder <- c(1,2,3,4,5);
+  }
+
+  # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+  # (a) Check if there is a cache lookup available.
+  # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   envir <- attr(this, ".env");
   cacheName <- paste("...$.lookup", name, sep=".");
   if (!is.null(envir) && exists(cacheName, envir=envir, inherits=FALSE)) {
@@ -1548,6 +1462,11 @@ setMethodS3("$", "Object", function(this, name) {
     lookup <- NULL;
   }
 
+
+  # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+  # (b) Otherwise search
+  # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+  static <- NULL;
   for (memberAccessor in memberAccessorOrder) {
     if (memberAccessor == 1) {
       if (is.null(attr(this, "disableGetMethods"))) {
@@ -1564,11 +1483,15 @@ setMethodS3("$", "Object", function(this, name) {
             capitalizedName <- name;
             substr(capitalizedName, start=1L, stop=1L) <- toupper(firstChar);
             getMethodNames <- paste("get", capitalizedName, ".", class(this), sep="");
+            static <- getStatic(static=static);
+            envirS <- environment(static);
             for (getMethodName in getMethodNames) {
-              if (exists(getMethodName, mode="function")) {
+              method <- .findS3Method(getMethodName, envir=envirS, mustExist=FALSE);
+##              if (exists(getMethodName, mode="function")) {
+##                method <- get(getMethodName, mode="function");
+              if (!is.null(method)) {
                 ref <- this;
                 attr(ref, "disableGetMethods") <- TRUE;
-                method <- get(getMethodName, mode="function");
                 # For caching purposes, if the field <name> is trying to be
                 # accessed, we do not want to call get<name>() again! If <name>
                 # is not a virtual field, the it has to be a real field, an
@@ -1611,12 +1534,17 @@ setMethodS3("$", "Object", function(this, name) {
     } else if (memberAccessor == 4) {
 
       # 4. Is it a static S3 method?
+      static <- getStatic(static=static);
+      envirS <- environment(static);
       methodNames <- paste(name, class(this), sep=".");
       for (methodName in methodNames) {
-        if (exists(methodName, mode="function")) {
+        method <- .findS3Method(methodName, envir=envirS, mustExist=FALSE);
+        if (!is.null(method)) {
+##        if (exists(methodName, mode="function")) {
           # Using explicit UseMethod() code
           code <- sprintf("function(...) \"%s\"(this, ...)", name);
           fcn <- eval(base::parse(text=code));
+##          environment(fcn) <- environment(method);
           lookup <- memberAccessor;
           attr(lookup, "memberAccessorOrder") <- memberAccessorOrder;
           attr(lookup, "fcn") <- fcn;
@@ -1627,10 +1555,7 @@ setMethodS3("$", "Object", function(this, name) {
     } else if (memberAccessor == 5) {
     
       # 5. Finally, if nothing is found, it might be that it is a static field
-      className <- class(this)[1];
-      # WAS: clazz <- get(className);
-      clazz <- .getClassByName(className);
-      static <- getStaticInstance(clazz);
+      static <- getStatic(static=static);
       static.envir <- attr(static, ".env");
       # For static method calls, e.g. Object$load, 'this' has no
       # environment assigned and therefore, for now, no static
@@ -1645,7 +1570,10 @@ setMethodS3("$", "Object", function(this, name) {
     }
   } # for (memberAccessor in memberAccessorOrder)
 
-  # 5. Otherwise, return NULL.
+
+  # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+  # (c) If not found, return NULL.
+  # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   lookup <- -1;
   attr(lookup, "memberAccessorOrder") <- memberAccessorOrder;
   assign(cacheName, lookup, envir=envir);
@@ -1721,10 +1649,24 @@ setMethodS3("$", "Object", function(this, name) {
 # \keyword{methods}
 #*/###########################################################################
 setMethodS3("$<-", "Object", function(this, name, value) {
-  memberAccessorOrder <- attr(this, ".memberAccessorOrder");
-  if (is.null(memberAccessorOrder))
-    memberAccessorOrder <- c(1,2,3,4,5);
+  # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+  # Local functions
+  # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+  getStatic <- function(static=NULL) {
+    if (!is.null(static)) return(static);
+    className <- class(this)[1];
+    clazz <- .getClassByName(className, envir=environment(this));
+    static <- getStaticInstance(clazz);
+    static;
+  } # getStatic()
 
+
+  memberAccessorOrder <- attr(this, ".memberAccessorOrder");
+  if (is.null(memberAccessorOrder)) {
+    memberAccessorOrder <- c(1,2,3,4,5);
+  }
+
+  static <- NULL;
   for (memberAccessor in memberAccessorOrder) {
     if (memberAccessor == 1) {
       # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
@@ -1744,11 +1686,16 @@ setMethodS3("$<-", "Object", function(this, name, value) {
             capitalizedName <- name;
             substr(capitalizedName,1,1) <- toupper(firstChar);
             setMethodNames <- paste("set", capitalizedName, ".", class(this), sep="");
+            static <- getStatic(static=static);
+            envirS <- environment(static);
             for (setMethodName in setMethodNames) {
-              if (exists(setMethodName, mode="function")) {
+              method <- .findS3Method(setMethodName, envir=envirS, mustExist=FALSE);
+              if (!is.null(method)) {
+##              if (exists(setMethodName, mode="function")) {
+##                method <- get(setMethodName, mode="function");
                 ref <- this;
                 attr(ref, "disableSetMethods") <- TRUE;
-                get(setMethodName, mode="function")(ref, value);
+                method(ref, value);
                 return(invisible(this));
               }
             }
@@ -1779,10 +1726,7 @@ setMethodS3("$<-", "Object", function(this, name, value) {
       # Search for a static <name> field
       # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
       # 4. If not, it might be that it is a static field
-      className <- class(this)[1];
-      # WAS: clazz <- get(className);
-      clazz <- .getClassByName(className);
-      static <- getStaticInstance(clazz);
+      static <- getStatic(static=static);
       static.envir <- attr(static, ".env");
       # For static method calls, e.g. Object$load, 'this' has no
       # environment assigned and therefore, for now, no static
@@ -1895,16 +1839,18 @@ setMethodS3("novirtual", "Object", function(this, ...) {
 
 
 setMethodS3("callSuperMethodS3", "default", function(this, methodName, ..., nbrOfClassesAbove=1) {
-  if (nbrOfClassesAbove < 0)
+  if (nbrOfClassesAbove < 0) {
     throw("Argument 'nbrOfClassesAbove' is negative.");
+  }
 
   classes <- class(this);
   nbrOfClassesAbove <- min(nbrOfClassesAbove, length(classes))
   classes <- classes[-seq(length=nbrOfClassesAbove)];
-  if (length(classes) == 0)
-    methods <- methodName
-  else
+  if (length(classes) == 0) {
+    methods <- methodName;
+  } else {
     methods <- c(paste(methodName, classes, sep="."), methodName);
+  }
   availableMethods <- c(methods(methodName), methodName);
   for (method in methods) {
     if (is.element(method, availableMethods)) {
@@ -2133,8 +2079,9 @@ setMethodS3("clearCache", "Object", function(this, recursive=TRUE, ...) {
 setMethodS3("getFieldModifiers", "Object", function(this, ...) {
   env <- attr(this, ".env");
 
-  if (!exists("...modifiers", envir=env))
+  if (!exists("...modifiers", envir=env)) {
     return(list());
+  }
 
   get("...modifiers", envir=env);
 }, protected=TRUE)
@@ -2276,6 +2223,10 @@ setMethodS3("registerFinalizer", "Object", function(this, ...) {
 
 ############################################################################
 # HISTORY:
+# 2013-01-08
+# o BUG FIX: The hexadecimal string returned by as.character() for Object
+#   would contain the decimal value and not the hexadecimal one.
+# o Added argument 'format' to getInternalAddress() for Object.
 # 2012-12-18
 # o R CMD check for R devel no longer gives a NOTE about attach().
 # 2012-11-28
