@@ -1022,11 +1022,14 @@ setMethodS3("getDevelUrl", "Package", function(this, ...) {
 # @synopsis
 #
 # \arguments{
-#   \item{...}{Not used.}
+#   \item{as}{A @character string specifying the return format.}
+#   \item{include}{A @character @vector  specifying which person fields
+#     to include if returning a @character string.}
+#   \item{...}{Optional arguments passed to @see "utils::format.person".}
 # }
 #
 # \value{
-#   Returns a @character string.
+#   Returns a @character string or a @see "utils::person" object.
 # }
 #
 # \examples{
@@ -1040,8 +1043,41 @@ setMethodS3("getDevelUrl", "Package", function(this, ...) {
 #   @seeclass
 # }
 #*/#########################################################################
-setMethodS3("getMaintainer", "Package", function(this, ...) {
-  getDescriptionFile(this, fields="Maintainer");
+setMethodS3("getMaintainer", "Package", function(this, as=c("character", "person"), include=c("given", "family"), ...) {
+  # Argument 'as':
+  as <- match.arg(as);
+
+  persons <- getDescriptionFile(this, fields=c("Authors@R", "Maintainer"));
+  persons <- persons[!is.na(persons)];
+  if (length(persons) == 0L) {
+    return(NA);
+  }
+
+  persons <- persons[1L];
+  key <- names(persons)[1L];
+
+  # Parse?
+  if (key == "Authors@R") {
+    personsP <- eval(parse(text=persons));
+    # Find creators
+    isCreator <- sapply(personsP, FUN=function(p) is.element("cre", p$role));
+    if (any(isCreator)) {
+       personsP <- personsP[isCreator];
+    }
+    persons <- format(personsP, include=include, ...);
+  } else {
+    personsP <- NULL;
+  }
+
+  if (as == "character") {
+    return(persons);
+  }
+
+  if (is.null(personsP)) {
+    personsP <- as.person(persons);
+  }
+
+  personsP;
 })
 
 
@@ -1058,11 +1094,14 @@ setMethodS3("getMaintainer", "Package", function(this, ...) {
 # @synopsis
 #
 # \arguments{
-#   \item{...}{Not used.}
+#   \item{as}{A @character string specifying the return format.}
+#   \item{include}{A @character @vector  specifying which person fields
+#     to include if returning a @character string.}
+#   \item{...}{Optional arguments passed to @see "utils::format.person".}
 # }
 #
 # \value{
-#   Returns a @character string.
+#   Returns a @character string or a @see "utils::person" object.
 # }
 #
 # \examples{
@@ -1076,8 +1115,35 @@ setMethodS3("getMaintainer", "Package", function(this, ...) {
 #   @seeclass
 # }
 #*/#########################################################################
-setMethodS3("getAuthor", "Package", function(this, ...) {
-  getDescriptionFile(this, fields="Author");
+setMethodS3("getAuthor", "Package", function(this, as=c("character", "person"), include=c("given", "family"), ...) {
+  # Argument 'as':
+  as <- match.arg(as);
+
+  persons <- getDescriptionFile(this, fields=c("Authors@R", "Author"));
+  persons <- persons[!is.na(persons)];
+  if (length(persons) == 0L) {
+    return(NA);
+  }
+  persons <- persons[1L];
+  key <- names(persons)[1L];
+
+  # Parse?
+  if (key == "Authors@R") {
+    personsP <- eval(parse(text=persons));
+    persons <- format(personsP, include=include, ...);
+  } else {
+    personsP <- NULL;
+  }
+
+  if (as == "character") {
+    return(persons);
+  }
+
+  if (is.null(personsP)) {
+    personsP <- as.person(persons);
+  }
+
+  personsP;
 })
 
 
@@ -1649,6 +1715,10 @@ setMethodS3("update", "Package", function(object, contribUrl=c(getContribUrl(thi
 
 ############################################################################
 # HISTORY:
+# 2013-03-08
+# o Now getAuthor() for Package uses the 'Authors@R' field of DESCRIPTION
+#   and if not found then the 'Author' field.  In addition, using argument
+#   'as="person"' with parse and return the authors list as 'person' object.
 # 2012-12-28
 # o Replaced all data.class(obj) with class(obj)[1].
 # 2012-12-19
